@@ -9,6 +9,8 @@
 #include <assert.h>
 #include <cups/cups.h>
 #include <cups/ppd.h>
+#include <iomanip>
+#include <sstream>
 #include <memory>
 
 #include "pdftopdf_processor.h"
@@ -402,8 +404,27 @@ void getParameters(ppd_file_t *ppd,int num_options,cups_option_t *options,Proces
     param.reverse=ppdDefaultOrder(ppd);
   }
 
-  // TODO: pageLabel  (not used)
-  // param.pageLabel=cupsGetOption("page-label",num_options,options);  // strdup?
+  std::string rawlabel;
+  char *classification = getenv("CLASSIFICATION");
+  if (classification)
+    rawlabel.append (classification);
+
+  if ( (val=cupsGetOption("page-label", num_options, options)) != NULL) {
+    if (!rawlabel.empty())
+      rawlabel.append (" - ");
+    rawlabel.append(cupsGetOption("page-label",num_options,options));
+  }
+
+  std::ostringstream cookedlabel;
+  for (std::string::iterator it = rawlabel.begin();
+       it != rawlabel.end ();
+       ++it) {
+    if (*it < 32 || *it > 126)
+      cookedlabel << "\\" << std::oct << std::setfill('0') << std::setw(3) << (unsigned int) *it;
+    else
+      cookedlabel.put (*it);
+  }
+  param.pageLabel = cookedlabel.str ();
 
   if ( (val=cupsGetOption("page-set",num_options,options)) != NULL) {
     if (strcasecmp(val,"even")==0) {
@@ -534,17 +555,7 @@ void calculate(ppd_file_t *ppd,ProcessingParameters &param) // {{{
     }
   }
 
-  if (param.numCopies==1) {
-    // collate is not needed
-    param.collate=false; // does not make a big difference for us
-  }
-/* TODO? instead:
-  if (...numOutputPages==1 [after nup,evenDuplex!]) {
-    param.collate=false; // does not make a big difference for us
-  }
-*/
-
-#if 0    // for now
+#if 1    // for now
   // enable hardware copy generation
   if (ppd) {
     if (!ppd->manual_copies) {
@@ -555,22 +566,29 @@ void calculate(ppd_file_t *ppd,ProcessingParameters &param) // {{{
       param.deviceCopies=1;
     }
   }
-#endif 
+#endif
 
   setFinalPPD(ppd,param);
 
-  // check collate device, with current ppd settings
-  if (param.collate) {
-    if (param.deviceCopies==1) { // e.g. ppd->manual_copies
-      param.deviceCollate=false;
-    } else {
-      param.deviceCollate=printerWillCollate(ppd);
-    }
+  if ( (param.numCopies==1)&&(param.deviceCopies==1) ) {
+    // collate is never needed for a single page
+    param.collate=false; // (this does not make a big difference for us)
+    param.deviceCollate=false;
+  } else if ( (param.deviceCopies==1)&&(param.duplex) ) { // i.e. (numCopies>1), in software
+    // duplex printing of multiple software copies:
+    // collate + evenDuplex must be forced to prevent copies on the backsides
+    param.collate=true;
+    param.deviceCollate=false; // either (!ppd) or (ppd->manual_copies)
+  } else if (param.collate) { // collate requested by user
+    // check collate device, with current/final(!) ppd settings
+    param.deviceCollate=printerWillCollate(ppd);
+  } else { // (!param.collate)
+    param.deviceCollate=false;
+  }
 
-    if (!param.deviceCollate) {
-      ppdMarkOption(ppd,"Collate","False"); // disable any hardware-collate
-      param.evenDuplex=true; // software collate always needs fillers
-    }
+  if ( (param.collate)&&(!param.deviceCollate) ) { // software collate
+    ppdMarkOption(ppd,"Collate","False"); // disable any hardware-collate (in JCL)
+    param.evenDuplex=true; // fillers always needed
   }
 
   if (!param.duplex) {
