@@ -26,6 +26,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  pdf to raster filter
 */
 
+#include <cups/cups.h>
+#if (CUPS_VERSION_MAJOR > 1) || (CUPS_VERSION_MINOR > 6)
+#define HAVE_CUPS_1_7 1
+#endif
+
 #define USE_CMS
 
 #include <config.h>
@@ -41,13 +46,13 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "PDFDoc.h"
 #include "SplashOutputDev.h"
 #include "GfxState.h"
-#include <cups/cups.h>
 #include <cups/ppd.h>
 #include <stdarg.h>
 #include "PDFError.h"
 #include "GlobalParams.h"
 #include <cups/raster.h>
 #include <cupsfilters/image.h>
+#include <cupsfilters/raster.h>
 #include <splash/SplashTypes.h>
 #include <splash/SplashBitmap.h>
 #include <strings.h>
@@ -185,11 +190,20 @@ namespace {
 }
 
 #if POPPLER_VERSION_MAJOR > 0 || POPPLER_VERSION_MINOR >= 19
+#if POPPLER_VERSION_MAJOR > 0 || POPPLER_VERSION_MINOR >= 23
+void CDECL myErrorFun(void *data, ErrorCategory category,
+    Goffset pos, char *msg)
+#else
 void CDECL myErrorFun(void *data, ErrorCategory category,
     int pos, char *msg)
+#endif
 {
   if (pos >= 0) {
+#if POPPLER_VERSION_MAJOR > 0 || POPPLER_VERSION_MINOR >= 23
+    fprintf(stderr, "ERROR (%lld): ", pos);
+#else
     fprintf(stderr, "ERROR (%d): ", pos);
+#endif
   } else {
     fprintf(stderr, "ERROR: ");
   }
@@ -357,80 +371,89 @@ static void parseOpts(int argc, char **argv)
   }
 
   ppd = ppdOpenFile(getenv("PPD"));
-  if (ppd == NULL) {
-    pdfError(-1,const_cast<char *>("PPD file is not specified"));
-    exit(1);
-  }
-  ppdMarkDefaults(ppd);
+  if (ppd == NULL)
+    fprintf(stderr, "DEBUG: PPD file is not specified.\n");
+  if (ppd)
+    ppdMarkDefaults(ppd);
   options = NULL;
   num_options = cupsParseOptions(argv[5],0,&options);
-  cupsMarkOptions(ppd,num_options,options);
-  handleRqeuiresPageRegion();
-  cupsRasterInterpretPPD(&header,ppd,num_options,options,0);
-  attr = ppdFindAttr(ppd,"pdftorasterRenderingIntent",NULL);
-  if (attr != NULL && attr->value != NULL) {
-    if (strcasecmp(attr->value,"PERCEPTUAL") != 0) {
-      renderingIntent = INTENT_PERCEPTUAL;
-    } else if (strcasecmp(attr->value,"RELATIVE_COLORIMETRIC") != 0) {
-      renderingIntent = INTENT_RELATIVE_COLORIMETRIC;
-    } else if (strcasecmp(attr->value,"SATURATION") != 0) {
-      renderingIntent = INTENT_SATURATION;
-    } else if (strcasecmp(attr->value,"ABSOLUTE_COLORIMETRIC") != 0) {
-      renderingIntent = INTENT_ABSOLUTE_COLORIMETRIC;
-    }
-  }
-  if (header.Duplex) {
-    /* analyze options relevant to Duplex */
-    const char *backside = "";
-    /* APDuplexRequiresFlippedMargin */
-    enum {
-      FM_NO, FM_FALSE, FM_TRUE
-    } flippedMargin = FM_NO;
-
-    attr = ppdFindAttr(ppd,"cupsBackSide",NULL);
+  if (ppd) {
+    cupsMarkOptions(ppd,num_options,options);
+    handleRqeuiresPageRegion();
+    cupsRasterInterpretPPD(&header,ppd,num_options,options,0);
+    attr = ppdFindAttr(ppd,"pdftorasterRenderingIntent",NULL);
     if (attr != NULL && attr->value != NULL) {
-      ppd->flip_duplex = 0;
-      backside = attr->value;
-    } else if (ppd->flip_duplex) {
-      backside = "Rotated"; /* compatible with Max OS and GS 8.71 */
+      if (strcasecmp(attr->value,"PERCEPTUAL") != 0) {
+	renderingIntent = INTENT_PERCEPTUAL;
+      } else if (strcasecmp(attr->value,"RELATIVE_COLORIMETRIC") != 0) {
+	renderingIntent = INTENT_RELATIVE_COLORIMETRIC;
+      } else if (strcasecmp(attr->value,"SATURATION") != 0) {
+	renderingIntent = INTENT_SATURATION;
+      } else if (strcasecmp(attr->value,"ABSOLUTE_COLORIMETRIC") != 0) {
+	renderingIntent = INTENT_ABSOLUTE_COLORIMETRIC;
+      }
     }
+    if (header.Duplex) {
+      /* analyze options relevant to Duplex */
+      const char *backside = "";
+      /* APDuplexRequiresFlippedMargin */
+      enum {
+	FM_NO, FM_FALSE, FM_TRUE
+      } flippedMargin = FM_NO;
 
-    attr = ppdFindAttr(ppd,"APDuplexRequiresFlippedMargin",NULL);
-    if (attr != NULL && attr->value != NULL) {
-      if (strcasecmp(attr->value,"true") == 0) {
-        flippedMargin = FM_TRUE;
-      } else {
-        flippedMargin = FM_FALSE;
+      attr = ppdFindAttr(ppd,"cupsBackSide",NULL);
+      if (attr != NULL && attr->value != NULL) {
+	ppd->flip_duplex = 0;
+	backside = attr->value;
+      } else if (ppd->flip_duplex) {
+	backside = "Rotated"; /* compatible with Max OS and GS 8.71 */
+      }
+
+      attr = ppdFindAttr(ppd,"APDuplexRequiresFlippedMargin",NULL);
+      if (attr != NULL && attr->value != NULL) {
+	if (strcasecmp(attr->value,"true") == 0) {
+	  flippedMargin = FM_TRUE;
+	} else {
+	  flippedMargin = FM_FALSE;
+	}
+      }
+      if (strcasecmp(backside,"ManualTumble") == 0 && header.Tumble) {
+	swap_image_x = swap_image_y = true;
+	swap_margin_x = swap_margin_y = true;
+	if (flippedMargin == FM_TRUE) {
+	  swap_margin_y = false;
+	}
+      } else if (strcasecmp(backside,"Rotated") == 0 && !header.Tumble) {
+	swap_image_x = swap_image_y = true;
+	swap_margin_x = swap_margin_y = true;
+	if (flippedMargin == FM_TRUE) {
+	  swap_margin_y = false;
+	}
+      } else if (strcasecmp(backside,"Flipped") == 0) {
+	if (header.Tumble) {
+	  swap_image_x = true;
+	  swap_margin_x = swap_margin_y = true;
+	} else {
+	  swap_image_y = true;
+	}
+	if (flippedMargin == FM_FALSE) {
+	  swap_margin_y = !swap_margin_y;
+	}
       }
     }
-    if (strcasecmp(backside,"ManualTumble") == 0 && header.Tumble) {
-      swap_image_x = swap_image_y = true;
-      swap_margin_x = swap_margin_y = true;
-      if (flippedMargin == FM_TRUE) {
-        swap_margin_y = false;
-      }
-    } else if (strcasecmp(backside,"Rotated") == 0 && !header.Tumble) {
-      swap_image_x = swap_image_y = true;
-      swap_margin_x = swap_margin_y = true;
-      if (flippedMargin == FM_TRUE) {
-        swap_margin_y = false;
-      }
-    } else if (strcasecmp(backside,"Flipped") == 0) {
-      if (header.Tumble) {
-        swap_image_x = true;
-        swap_margin_x = swap_margin_y = true;
-      } else {
-        swap_image_y = true;
-      }
-      if (flippedMargin == FM_FALSE) {
-        swap_margin_y = !swap_margin_y;
-      }
+    if (getColorProfilePath(ppd,&profilePath)) {
+      /* ICCProfile is specified */
+      colorProfile = cmsOpenProfileFromFile(profilePath.getCString(),"r");
     }
+  } else
+#ifdef HAVE_CUPS_1_7
+    cupsRasterParseIPPOptions(&header,num_options,options,1,1);
+#else
+  {
+    fprintf(stderr, "ERROR: No PPD file specified.\n");
+    exit(1);
   }
-  if (getColorProfilePath(ppd,&profilePath)) {
-    /* ICCProfile is specified */
-    colorProfile = cmsOpenProfileFromFile(profilePath.getCString(),"r");
-  }
+#endif /* HAVE_CUPS_1_7 */
 }
 
 static void parsePDFTOPDFComment(FILE *fp)
@@ -1548,48 +1571,23 @@ static void outPage(PDFDoc *doc, Catalog *catalog, int pageNo,
 
   memset(paperdimensions, 0, sizeof(paperdimensions));
   memset(margins, 0, sizeof(margins));
-  for (i = ppd->num_sizes, size = ppd->sizes;
-       i > 0;
-       i --, size ++) {
-    /* Skip page sizes which conflict with settings of the other options */
-    /* TODO XXX */
-    /* Find size of document's page under the PPD page sizes */
-    if (fabs(header.PageSize[1] - size->length) < 5.0 &&
-	fabs(header.PageSize[0] - size->width) < 5.0)
-      break;
-  }
-  if (i > 0) {
-    /*
-     * Standard size...
-     */
-    fprintf(stderr, "DEBUG: size = %s\n", size->name);
-    landscape = 0;
-    paperdimensions[0] = size->width;
-    paperdimensions[1] = size->length;
-    margins[0] = size->left;
-    margins[1] = size->bottom;
-    margins[2] = size->width - size->right;
-    margins[3] = size->length - size->top;
-    strncpy(header.cupsPageSizeName, size->name, 64);
-  } else {
-    /*
-     * No matching portrait size; look for a matching size in
-     * landscape orientation...
-     */
-
+  if (ppd) {
     for (i = ppd->num_sizes, size = ppd->sizes;
 	 i > 0;
-	 i --, size ++)
-      if (fabs(header.PageSize[0] - size->length) < 5.0 &&
-	  fabs(header.PageSize[1] - size->width) < 5.0)
+	 i --, size ++) {
+      /* Skip page sizes which conflict with settings of the other options */
+      /* TODO XXX */
+      /* Find size of document's page under the PPD page sizes */
+      if (fabs(header.PageSize[1] - size->length) < 5.0 &&
+	  fabs(header.PageSize[0] - size->width) < 5.0)
 	break;
-
+    }
     if (i > 0) {
       /*
-       * Standard size in landscape orientation...
+       * Standard size...
        */
-      fprintf(stderr, "DEBUG: landscape size = %s\n", size->name);
-      landscape = 1;
+      fprintf(stderr, "DEBUG: size = %s\n", size->name);
+      landscape = 0;
       paperdimensions[0] = size->width;
       paperdimensions[1] = size->length;
       margins[0] = size->left;
@@ -1599,17 +1597,53 @@ static void outPage(PDFDoc *doc, Catalog *catalog, int pageNo,
       strncpy(header.cupsPageSizeName, size->name, 64);
     } else {
       /*
-       * Custom size...
+       * No matching portrait size; look for a matching size in
+       * landscape orientation...
        */
-      fprintf(stderr, "DEBUG: size = Custom\n");
-      landscape = 0;
-      paperdimensions[1] = size->length;
-      for (i = 0; i < 2; i ++)
-	paperdimensions[i] = header.PageSize[i];
-      for (i = 0; i < 4; i ++)
-	margins[i] = ppd->custom_margins[i];
-      header.cupsPageSizeName[0] = '\0';
+
+      for (i = ppd->num_sizes, size = ppd->sizes;
+	   i > 0;
+	   i --, size ++)
+	if (fabs(header.PageSize[0] - size->length) < 5.0 &&
+	    fabs(header.PageSize[1] - size->width) < 5.0)
+	  break;
+
+      if (i > 0) {
+	/*
+	 * Standard size in landscape orientation...
+	 */
+	fprintf(stderr, "DEBUG: landscape size = %s\n", size->name);
+	landscape = 1;
+	paperdimensions[0] = size->width;
+	paperdimensions[1] = size->length;
+	margins[0] = size->left;
+	margins[1] = size->bottom;
+	margins[2] = size->width - size->right;
+	margins[3] = size->length - size->top;
+	strncpy(header.cupsPageSizeName, size->name, 64);
+      } else {
+	/*
+	 * Custom size...
+	 */
+	fprintf(stderr, "DEBUG: size = Custom\n");
+	landscape = 0;
+	paperdimensions[1] = size->length;
+	for (i = 0; i < 2; i ++)
+	  paperdimensions[i] = header.PageSize[i];
+	for (i = 0; i < 4; i ++)
+	  margins[i] = ppd->custom_margins[i];
+	header.cupsPageSizeName[0] = '\0';
+      }
     }
+  } else {
+    for (i = 0; i < 2; i ++)
+      paperdimensions[i] = header.PageSize[i];
+    for (i = 0; i < 4; i ++)
+      margins[i] = 0.0;
+    /*margins[0] = 0.0;
+    margins[1] = 0.0;
+    margins[2] = header.PageSize[0];
+    margins[3] = header.PageSize[1];*/
   }
 
   if (header.Duplex && (pageNo & 1) == 0) {
@@ -1762,19 +1796,15 @@ int main(int argc, char *argv[]) {
   if (argc == 6) {
     /* stdin */
     int fd;
-    Object obj;
-    BaseStream *str;
-    FILE *fp;
+    char name[BUFSIZ];
     char buf[BUFSIZ];
     int n;
 
-    fd = cupsTempFd(buf,sizeof(buf));
+    fd = cupsTempFd(name,sizeof(name));
     if (fd < 0) {
       pdfError(-1,const_cast<char *>("Can't create temporary file"));
       exit(1);
     }
-    /* remove name */
-    unlink(buf);
 
     /* copy stdin to the tmp file */
     while ((n = read(0,buf,BUFSIZ)) > 0) {
@@ -1784,23 +1814,10 @@ int main(int argc, char *argv[]) {
 	exit(1);
       }
     }
-    if (lseek(fd,0,SEEK_SET) < 0) {
-        pdfError(-1,const_cast<char *>("Can't rewind temporary file"));
-        close(fd);
-	exit(1);
-    }
-
-    if ((fp = fdopen(fd,"rb")) == 0) {
-        pdfError(-1,const_cast<char *>("Can't fdopen temporary file"));
-        close(fd);
-	exit(1);
-    }
-
-    obj.initNull();
-    parsePDFTOPDFComment(fp);
-    rewind(fp);
-    str = new FileStream(fp,0,gFalse,0,&obj);
-    doc = new PDFDoc(str);
+    close(fd);
+    doc = new PDFDoc(new GooString(name));
+    /* remove name */
+    unlink(name);
   } else {
     GooString *fileName = new GooString(argv[6]);
     /* argc == 7 filenmae is specified */
