@@ -24,11 +24,15 @@
 #include <ctype.h>
 #include <errno.h>
 #include <ifaddrs.h>
+#if defined(__OpenBSD__)
+#include <sys/socket.h>
+#endif /* __OpenBSD__ */
 #include <net/if.h>
 #include <netinet/in.h>
 #include <resolv.h>
 #include <stdio.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <assert.h>
 #include <stdlib.h>
 #include <time.h>
@@ -116,6 +120,7 @@ static unsigned int BrowseTimeout = 300;
 static uint16_t BrowsePort = 631;
 static char **BrowsePoll = NULL;
 static size_t NumBrowsePoll = 0;
+static char *DomainSocket = NULL;
 
 static int debug = 0;
 
@@ -400,6 +405,8 @@ gboolean handle_cups_queues(gpointer unused) {
 	  p->timeout = current_time + TIMEOUT_RETRY;
 	  break;
 	}
+	if (response)
+	  ippDelete(response);
 
 	/* No jobs, not default printer, remove the CUPS queue */
 	request = ippNewRequest(CUPS_DELETE_PRINTER);
@@ -632,6 +639,9 @@ void generate_local_queue(const char *host,
 	   ignore this remote printer */
 	debug_printf("cups-browsed: %s also taken, printer ignored.\n",
 		     local_queue_name);
+	free (backup_queue_name);
+	free (remote_host);
+	cupsFreeDests(num_dests, dests);
 	return;
       }
     }
@@ -663,6 +673,10 @@ void generate_local_queue(const char *host,
       p->uri[3] = 's';
       p->status = STATUS_TO_BE_CREATED;
       p->timeout = time(NULL) + TIMEOUT_IMMEDIATELY;
+      p->host = strdup(remote_host);
+      p->service_name = strdup(name);
+      p->type = strdup(type);
+      p->domain = strdup(domain);
       debug_printf("cups-browsed: Upgrading printer %s (Host: %s) to IPPS. New URI: %s\n",
 		   p->name, p->host, p->uri);
 
@@ -680,6 +694,14 @@ void generate_local_queue(const char *host,
       }
 
     }
+    if (p->host[0] == '\0')
+      p->host = strdup(remote_host);
+    if (p->service_name[0] == '\0' && name)
+      p->service_name = strdup(name);
+    if (p->type[0] == '\0' && type)
+      p->type = strdup(type);
+    if (p->domain[0] == '\0' && domain)
+      p->domain = strdup(domain);
 
   } else {
 
@@ -702,6 +724,9 @@ void generate_local_queue(const char *host,
     free (uri);
   }
 
+  free (backup_queue_name);
+  free (remote_host);
+
   if (p)
     debug_printf("cups-browsed: Bonjour IDs: Service name: \"%s\", "
 		 "Service type: \"%s\", Domain: \"%s\"\n",
@@ -968,23 +993,6 @@ found_cups_printer (const char *remote_host, const char *uri,
   debug_printf("cups-browsed: browsed queue name is %s\n",
 	       local_resource + 9);
 
-  /* Does the host need resolving? */
-  if (host[strspn (host, "0123456789.")] == '\0') {
-    /* Yes. Resolve it. */
-    struct addrinfo hints, *addr;
-    memset(&hints, 0, sizeof(struct addrinfo));
-    hints.ai_family = AF_UNSPEC;
-    hints.ai_socktype = SOCK_DGRAM;
-    hints.ai_flags = 0;
-    hints.ai_protocol = 0;
-    if (!getaddrinfo (host, NULL, &hints, &addr)) {
-      getnameinfo (addr->ai_addr, addr->ai_addrlen,
-		   host, sizeof(host),
-		   NULL, 0, 0);
-      freeaddrinfo (addr);
-    }
-  }
-
   generate_local_queue(host, port, local_resource, info ? info : "", "", "");
 }
 
@@ -1460,7 +1468,32 @@ browse_poll (gpointer data)
   ipp_attribute_t *attr;
   http_t *conn;
   int port = BrowsePort;
-  char *colon;
+  char *colon,*slash;
+  int major = 0;
+  int minor = 0;
+
+  slash = strchr (server, '/');
+  if (slash) {
+    *slash++ = '\0';
+    if (!strcmp(slash, "version=1.0")) {
+      major = 1;
+      minor = 0;
+    } else if (!strcmp(slash, "version=1.1")) {
+      major = 1;
+      minor = 1;
+    } else if (!strcmp(slash, "version=2.0")) {
+      major = 2;
+      minor = 0;
+    } else if (!strcmp(slash, "version=2.1")) {
+      major = 2;
+      minor = 1;
+    } else if (!strcmp(slash, "version=2.2")) {
+      major = 2;
+      minor = 2;
+    } else {
+      debug_printf ("ignoring unknown server option: %s\n", slash);
+    }
+  }
 
   debug_printf ("cups-browsed: browse polling %s\n", server);
 
@@ -1483,6 +1516,10 @@ browse_poll (gpointer data)
   }
 
   request = ippNewRequest(CUPS_GET_PRINTERS);
+  if (major > 0) {
+    debug_printf("cups-browsed: setting IPP version %d.%d\n", major, minor);
+    ippSetVersion (request, major, minor);
+  }
 
   ippAddStrings (request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
 		 "requested-attributes", sizeof (rattrs) / sizeof (rattrs[0]),
@@ -1547,8 +1584,11 @@ fail:
   if (conn)
     httpClose (conn);
 
+  if (server)
+    free (server);
+
   /* Call a new timeout handler so that we run again */
-  g_timeout_add_seconds (BrowseInterval, browse_poll, server);
+  g_timeout_add_seconds (BrowseInterval, browse_poll, data);
 
   /* Stop this timeout handler, we called a new one */
   return FALSE;
@@ -1682,10 +1722,14 @@ read_configuration (const char *filename)
 	debug_printf("cups-browsed: Adding BrowsePoll server: %s\n", value);
 	BrowsePoll[NumBrowsePoll++] = strdup (value);
       }
-    } else if (!strcasecmp(line, "BrowseAllow") && value)
+    } else if (!strcasecmp(line, "BrowseAllow") && value) {
       if (read_browseallow_value (value))
 	debug_printf ("cups-browsed: BrowseAllow value \"%s\" not understood\n",
 		      value);
+    } else if (!strcasecmp(line, "DomainSocket") && value) {
+      if (value[0] != '\0')
+	DomainSocket = strdup(value);
+    }
   }
 
   cupsFileClose(fp);
@@ -1718,6 +1762,23 @@ int main(int argc, char*argv[]) {
   /* Read in cups-browsed.conf */
   read_configuration (NULL);
 
+  /* Set the CUPS_SERVER environment variable to assure that cups-browsed
+     always works with the local CUPS daemon and never with a remote one
+     specified by a client.conf file */
+#ifdef CUPS_DEFAULT_DOMAINSOCKET
+  if (DomainSocket == NULL)
+    DomainSocket = CUPS_DEFAULT_DOMAINSOCKET;
+#endif
+  if (DomainSocket != NULL) {
+    struct stat sockinfo;               /* Domain socket information */
+    if (!stat(DomainSocket, &sockinfo) &&
+        (sockinfo.st_mode & S_IRWXO) == S_IRWXO)
+      setenv("CUPS_SERVER", DomainSocket, 1);
+    else
+      setenv("CUPS_SERVER", "localhost", 1);
+  } else
+    setenv("CUPS_SERVER", "localhost", 1);
+
   if (BrowseLocalProtocols & BROWSE_DNSSD) {
     fprintf(stderr, "Local support for DNSSD not implemented\n");
     BrowseLocalProtocols &= ~BROWSE_DNSSD;
@@ -1918,13 +1979,13 @@ int main(int argc, char*argv[]) {
   }
 
   if (BrowsePoll) {
-    char **server;
-    for (server = BrowsePoll;
-	 *server;
-	 server++) {
+    size_t index;
+    for (index = 0;
+	 index < NumBrowsePoll;
+	 index++) {
       debug_printf ("cups-browsed: will browse poll %s every %ds\n",
-		    *server, BrowseInterval);
-      g_idle_add (browse_poll, *server);
+		    BrowsePoll[index], BrowseInterval);
+      g_idle_add (browse_poll, BrowsePoll[index]);
     }
   }
 
